/*********************************************************************

 Author        : ADI - Apps            www.analog.com/MicroConverter

 Date          : October 2006

 File          : XIRQ0.c

 Hardware      : Applicable to ADuC7020/ADuC7024/ADuC7026
                 Currently targetting the ADuC7024

 Description   : Complement port 3 in a loop.
       When XIRQ0 is pressed, complement P4.2


 Notes         : This program includes example Data Abort, Prefetch Abort
                 and Undefined instruction handlers.

                 For robust programs, this functions should contain code
                 that will allow resumption of user code execution in case
                 of fault. In this example, shell functions are provided
                 with infinte loops.




*********************************************************************/

#include "ioaduc7024.h"             //    Include ADuC7024 Header File
#include <intrinsics.h>


__irq __arm void irq_handler(void);       // IRQ Function Prototype


void delay(int);


// IMPORTANT - The amount of stack spaces allocated for the IRQ stack is
//             defined by _IRQ_STACK_SIZE in the linker script file. If you add
//             additional function calls or additional local variables then
//             you should verify that _IRQ_STACK_SIZE is still sufficiently
//             large to prevent overflow of the stack and corruption of data

__irq __arm void irq_handler(void)                      //      IRQ Handler
{
   GP4DAT ^= 0x04040000;            //      Complement P4.2
   while(GP0DAT & 0x00010){}        //      wait for XIRQ to be low again
   return ;
}

// IMPORTANT - The amount of stack spaces allocated for the UND stack is
//             defined by _UND_STACK_SIZE in the linker script file. If you add
//             additional function calls or additional local variables then
//             you should verify that _UND_STACK_SIZE is still sufficiently
//             large to prevent overflow of the stack and corruption of data
//
// hander must be compiler for arm. Using __irq for the function prologue,
// function should perform some useful operation rather than just simply
// returning to the instruction that caused the execption
__irq __arm void undef_handler(void)                      //      IRQ Handler
{
        while (1);
}

// IMPORTANT - The amount of stack spaces allocated for the ABT stack is
//             defined by _ABT_STACK_SIZE in the linker script file. If you add
//             additional function calls or additional local variables then
//             you should verify that _ABT_STACK_SIZE is still sufficiently
//             large to prevent overflow of the stack and corruption of data
//
// hander must be compiler for arm. Using __irq for the function prologue,
// function should perform some useful operation rather than just simply
// returning to the instruction that caused the execption
__irq __arm void prefetch_handler(void)                 //      prefetch Handler
{
        while (1);
}

// IMPORTANT - The amount of stack spaces allocated for the ABT stack is
//             defined by _ABT_STACK_SIZE in the linker script file. If you add
//             additional function calls or additional local variables then
//             you should verify that _ABT_STACK_SIZE is still sufficiently
//             large to prevent overflow of the stack and corruption of data
//
// hander must be compiler for arm. Using __irq for the function prologue,
// function should perform some useful operation rather than just simply
// returning to the instruction that caused the execption
__irq __arm void data_handler(void)                 //      data Handler
{
        while (1);
}

// IMPORTANT - The amount of stack spaces allocated for the FIQ stack is
//             defined by _FIQ_STACK_SIZE in the linker script file. If you add
//             additional function calls or additional local variables then
//             you should verify that _FIQ_STACK_SIZE is still sufficiently
//             large to prevent overflow of the stack and corruption of data
//
__fiq __arm void fiq_handler(void)                 //      fiq Handler
{
   // Replace this while (1) code with a proper handler for the fiq source
   while (1);
}

int main (void)
{

   IRQEN = XIRQ0_BIT;            // Enable XIRQ0 in IRQEnable

   GP0CON = 0x00;
   GP4DAT = 0x04000000;          // Configure P4.2 as output
   GP3DAT = 0xff000000;          // Configure port 3 as output

#ifdef USE_REMAP   // Have RAM at 0x0 instead of flash?
    REMAP = 0x1;
#endif

   __enable_interrupt();                           //      Enable interrupts
   while(1)
   {
      GP3DAT ^= 0x00ff0000;            //    Complement Port 3
      delay(900000);          // Delay to make blink visible
   }
}



void delay (int length)                                 //      Delay function
{
   while (length >=0)
      length--;
}


